function Paper_Riemannian_Algorithms()
% Reproduce the experiments in Wen Huang's Dissertation
% Test the performance of Riemannian algorithms for the Brockett problem on
% the Stiefel manifold.
% 
% By Wen Huang

    methods = 11;%[12, 9, 3, 6, 11];
    n1s = [16, 16, 32, 32, 64, 64, 128];%, 128, 256];
    n2s = [16, 32, 32, 64, 64, 128, 128];%, 256, 256];
    tab = zeros(6 * 2 + 5 * 3, length(n1s));
    seeds = [1, 2, 3, 4, 5];
    
    for j = 1 : length(n1s)
        row = 0;
        for i = 1 : length(methods)
            for k = 1 : length(seeds)
                [methods(i), n1s(j), n2s(j), seeds(k)]
                [X, F, G, T, timecost, nf, ng, nR, nH, nV, nVp, ave_time] = test_PhaseRetrieval(methods(i), n1s(j), n2s(j), seeds(k));
                if(i <= 2)
                    tab(row + 1, j) = tab(row + 1, j) + length(F);
                    tab(row + 2, j) = tab(row + 2, j) + nf;
                    tab(row + 3, j) = tab(row + 3, j) + ng;
                    tab(row + 4, j) = tab(row + 4, j) + nH;
                    tab(row + 5, j) = tab(row + 5, j) + F(end);
                    tab(row + 6, j) = tab(row + 6, j) + ave_time;
                else
                    tab(row + 1, j) = tab(row + 1, j) + length(F);
                    tab(row + 2, j) = tab(row + 2, j) + nf;
                    tab(row + 3, j) = tab(row + 3, j) + ng;
                    tab(row + 4, j) = tab(row + 4, j) + F(end);
                    tab(row + 5, j) = tab(row + 5, j) + ave_time;
                end
            end
            if(i <= 2)
                tab(row + 1, j) = tab(row + 1, j) / length(seeds);
                tab(row + 2, j) = tab(row + 2, j) / length(seeds);
                tab(row + 3, j) = tab(row + 3, j) / length(seeds);
                tab(row + 4, j) = tab(row + 4, j) / length(seeds);
                tab(row + 5, j) = tab(row + 5, j) / length(seeds);
                tab(row + 6, j) = tab(row + 6, j) / length(seeds);
                row = row + 6;
            else
                tab(row + 1, j) = tab(row + 1, j) / length(seeds);
                tab(row + 2, j) = tab(row + 2, j) / length(seeds);
                tab(row + 3, j) = tab(row + 3, j) / length(seeds);
                tab(row + 4, j) = tab(row + 4, j) / length(seeds);
                tab(row + 5, j) = tab(row + 5, j) / length(seeds);
                row = row + 5;
            end
        end
    end
    
    fout = fopen('Paper_Riemannian_Algorithms_tab.txt','w');
    for i = 1 : size(tab, 1)
        for j = 1 : size(tab, 2)
            if(mod(tab(i, j), 1) == 0)
                if(j < size(tab, 2))
                    fprintf(fout,'%d & ',tab(i, j));
                else
                    fprintf(fout,'%d',tab(i, j));
                end
            else
                tabstr = outputfloat(tab(i, j));
                if(j < size(tab, 2))
                    fprintf(fout,['$' tabstr '$ & ']);
                else
                    fprintf(fout,['$' tabstr '$']);
                end
            end
        end
        fprintf(fout,' \\\\\n');
    end
    fclose(fout);
    

%     figure('Position', [1, 1, 720, 480]);clf
%     set(0,'defaultaxesfontsize',10, ...
%        'defaultaxeslinewidth',0.7, ...
%        'defaultlinelinewidth',.8,'defaultpatchlinewidth',0.8);
%     set(0,'defaultlinemarkersize',10)
%     colors = ['k', 'r', 'b', 'm', 'g', 'c', 'y'];
%     subplot(2, 1, 1)
%     for i = 1 : 5
%         semilogy(1 : length(EstiefelGS{i}), EstiefelGS{i}, ['.' colors(i)]);
%         hold on
%     end
%     axis square
%     axis fill
%     axis tight
%     xlabel('iter.')
%     ylabel('|grad f|')
%     title('Brockett on the Stiefel manifold');
%     legend('RTR-Newton', 'RBFGS', 'RTR-SR1', 'RSD', 'RTR-SD', 'Location', 'NorthEast');
%     subplot(2, 1, 2)
%     for i = 1 : 5
%         semilogy(EstiefelTS{i}, EstiefelGS{i}, ['.' colors(i)]);
%         hold on
%     end
%     axis square
%     axis fill
%     axis tight
%     xlabel('time(second)')
%     ylabel('|grad f|')
%     title('Brockett on the Stiefel manifold');
%     legend('RTR-Newton', 'RBFGS', 'RTR-SR1', 'RSD', 'RTR-SD', 'Location', 'NorthEast');
% return;
end

function str = outputfloat(x)
    if(x <= 0)
        str = '';
        return;
    end
    p = log(x)/log(10);
    p = - ceil(-p);
    x = round(x * 10^(-p) * 100);
    x = x / 100;
    strx = sprintf('%3.2f', x);
    if(p ~= 0)
        str = [strx '_{' num2str(p) '}'];
    else
        str = strx;
    end
end

function [X, F, G, T, timecost, nf, ng, nR, nH, nV, nVp, ave_time] = test_PhaseRetrieval(method, n1, n2, seed)
% test file for test_PhaseRetrieval.
% 
% By Wen Huang
    fprintf('test for Phase Retrieval problem. \n');
    r = rand();
%     r = floor(r * 10000)
%     r = 1;
%     r = 943
r = seed;
    rand('state', r);
    randn('state', r);
%     n1 = 64;
%     n2 = 64;
    global count
    count = 0;
    n = n1 * n2;
    l = 6;
    m = l * n;
    k = 8;
    
    fprintf('In this case, n1 = %d, n2 = %d, n = %d, l = %d. b, and x0 are chosen randomly. Random seed is %d \n', n1, n2, n, l, r);
    
    metric = 1;

    %% generate masks and a true solution.
    global iscomplex
    iscomplex = 1;
    global isfixedrank
    isfixedrank = 0;
    masks = complex(randn(n1, n2, l), iscomplex * randn(n1, n2, l));
    phase = complex(randn(n1, n2), iscomplex * randn(n1, n2));
    phase = phase / norm(phase);
    b = zeros(m, 1);
    for i = 1 : l
        zi = reshape(fft2(phase .* masks(:, :, i)) / sqrt(n1 * n2), [], 1);
        b((i - 1) * n + 1 : i * n) = conj(zi) .* zi;
    end
%     bnorm = norm(b)
%     phasenorm = norm(phase, 'fro')
    
    %%
    initialtype = 0; % 0: random, 1: true solution, 2: modification of true solution
    x0eps = 0.1;
    if((initialtype == 1 || initialtype == 2) && k ~= 1)
        fprintf('k should be 1\n');
        return;
    end
    
    if(initialtype == 0)
        [x1.U, r] = qr(complex(randn(n, k), iscomplex * randn(n, k)), 0);
%         x1.D = spdiags(rand(k, 1), 0, k, k);
        x1.D = spdiags(ones(k, 1), 0, k, k);
    elseif(initialtype == 1)
        x1.U = reshape(phase, [], 1) / norm(reshape(phase, [], 1));
        x1.D = norm(reshape(phase, [], 1))^2;
    elseif(initialtype == 2)
        temp = phase + randn(n1, n2) * x0eps;
        x1.U = reshape(temp, [], 1) / norm(reshape(temp, [], 1));
        x1.D = norm(reshape(temp, [], 1))^2;
    end
    
    if(metric == 1 || metric == 2)
        x1.U = full(x1.U * sqrt(x1.D));
        if(method == 2 || method == 5)
            params.H0 = eye(n * k);
            params.B0 = eye(n * k);
        end
    end
    
    params.x0 = x1;
    params.error = 1e-6;
    params.max_t = inf;
    params.debug = 0;
    params.manifold = 2;
    params.m = 4;
    params.alpha = 1e-4;
    params.beta = 0.999;
    params.min_Delta = 1e-8;
    params.minstepsize = eps;
    params.maxstepsize = 200;
%     params.kappa = 0.9;
%     params.retraction = Retraction;
%     params.vector_transport = VT;
    params.Delta_bar = inf;
    params.Delta0 = 1;
    params.StopCriterion = 0;
    params.restart = 0;
    if(method == 11)
        params.linesearch = 3;
        params.beta = 0.01;
        params.initstepsize = 1;
    end
    delta = 0.9;
    times = 0;
    total_timecost = 0;
    while(total_timecost < 30 && times < 2)
        fprintf('%d: ', times);
        [X, F, G, T1, timecost, nf, ng, nR, nH, nV, nVp] = driver_PhaseRetrieval(b, masks, method, params, phase, metric, delta, 0, 0);
        if(times == 0)
            T = T1;
        else
            T = T + T1;
        end
        total_timecost = total_timecost + timecost;
        times = times + 1;
    end
    if(method == 13)
        return;
    end
    T = T / times;
    ave_time = total_timecost / times;
    fprintf('average time cost of %d times is %f. \n', times, total_timecost / times);
end
