function test_TFOCS_PhaseRetrieval
%% Tests the solvers on a simple constrained quadratic function
%{
    Solve:

    minimize_x  c'x + x'Dx/2
    subject to  ||x||_1 <= 10

    as an example of using TFOCS without the "SCD" interface
    (since the objective is smooth and we can project, there's 
     no need to smooth and solve the dual)

    It's also an example of using 2 objective functions

%}

%% Now, add in constraints
    r = rand();
    r = floor(r * 10000)
    r = 1;
    rand('state', r);
    randn('state', r);
    n1 = 64;
    n2 = 64;
    n = n1 * n2;
    l = 5;
    m = l * n;
    k = 4;

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% CONSTRUCT THE TEST PROBLEM %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

    %% generate masks and a true solution.
    global iscomplex
    iscomplex = 1;
    masks = complex(randn(n1, n2, l), iscomplex * randn(n1, n2, l));
    phase = complex(randn(n1, n2), iscomplex * randn(n1, n2));
    phase = phase / norm(phase);
    b = zeros(m, 1);
    for i = 1 : l
        zi = reshape(fft2(phase .* masks(:, :, i)) / sqrt(n1 * n2), [], 1);
        b((i - 1) * n + 1 : i * n) = conj(zi) .* zi;
    end
    bnorm = norm(b)
    phasenorm = norm(phase, 'fro')
    
    initialtype = 0; % 0: random, 1: true solution, 2: modification of true solution
    x0eps = 0.1;
    if((initialtype == 1 || initialtype == 2) && k ~= 1)
        fprintf('k should be 1\n');
        return;
    end
    
    if(initialtype == 0)
        [x1.U, r] = qr(complex(randn(n, k), iscomplex * randn(n, k)), 0);
%         x1.D = spdiags(rand(k, 1), 0, k, k);
        x1.D = spdiags(ones(k, 1), 0, k, k);
    elseif(initialtype == 1)
        x1.U = reshape(phase, [], 1) / norm(reshape(phase, [], 1));
        x1.D = norm(reshape(phase, [], 1))^2;
    elseif(initialtype == 2)
        temp = phase + randn(n1, n2) * x0eps;
        x1.U = reshape(temp, [], 1) / norm(reshape(temp, [], 1));
        x1.D = norm(reshape(temp, [], 1))^2;
    end
    x0 = full(x1.U * sqrt(x1.D));

    smoothF = @(x)f_gf(x, masks, b);
    
    
%%%%%%%%%%%%%%%%%%%%%
% ADD IN CONSTRAINT %
%%%%%%%%%%%%%%%%%%%%%
% projectorF      = proj_simplex(10); % x >=0, sum(x) = 10
% projectorF      = proj_l1(10);  % l1 ball, radius 10 (e.g. norm(x,1) <= 10)

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% SET UP THE TEST PARAMETERS %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    opts = [];
    opts.tol        = 1e-16;
    opts.maxits     = 300;
    opts.alg = 'N83_PR';
%     opts.stopCrit = inf;
    opts.stopFcn = @stopFcn;

%     [f1, gfv1] = f_gf(x0, masks, b);
%     gf1 = gfv1(eye(size(x0, 1)));
%     [n, k] = size(x0);
%     v = complex(randn(n, n), randn(n, n));
%     f2 = f_gf(sqrtm(x0 * x0' + 1e-8 * v), masks, b);
%     (f2 - f1) / 1e-8
%     real(gf1(:)'*v(:))
%     return

%%%%%%%%%%%%%%%%%
% RUN THE TESTS %
%%%%%%%%%%%%%%%%%
    [ x, out, optsOut ] = tfocs(smoothF, [], [], x0, opts );
return;

function stop = stopFcn(f, x)
    stop = f < 1e-4;
return;

function [f, gf] = f_gf(x, masks, b)
    [n1, n2, l] = size(masks);
    n = n1 * n2;
    k = size(x, 2);
    m = l * n;
    D = zeros(m, 1);
    ZY = zeros(m, k);
    for i = 1 : l
        sZqi = zeros(n, 1);
        for j = 1 : k
            temp = reshape(fft2(reshape(x(:, j), n1, n2) / sqrt(n1 * n2) .* masks(:, :, i)), [], 1);
            ZY((i - 1) * n + 1 : i * n, j) = reshape(temp, [], 1);
            sZqi = sZqi + conj(temp) .* temp;
        end
        D((i - 1) * n + 1 : i * n) = reshape(sZqi, [], 1) - b((i - 1) * n + 1 : i * n);
    end
    f = norm(D)^2 / norm(b, 'fro')^2;
    if(nargout > 1)
        gf = @(v)gfv(v, D, masks, b);
    end
return

function output = gfv(v, D, masks, b)
    [n1, n2, l] = size(masks);
    n = n1 * n2;
    s = size(v, 2);
    m = length(D);
    Zv = zeros(m, s);    
    for i = 1 : l
        for j = 1 : s
            temp = reshape(fft2(reshape(v(:, j), n1, n2) / sqrt(n1 * n2) .* masks(:, :, i)), [], 1);
            Zv((i - 1) * n + 1 : i * n, j) = reshape(temp, [], 1);
        end
    end
    
    DZv = spdiags(D, 0, m, m) * Zv;
    %% Z^H *DZv
    Y = zeros(n, s);
    for i = 1 : l
        temp = zeros(n, s);
        for j = 1 : s
            temp(:, j) = reshape(ifft2(reshape(DZv((i - 1) * n + 1 : i * n, j) * sqrt(n1 * n2), n1, n2)), [], 1);
        end
        temp = conj(spdiags(reshape(masks(:, :, i), [], 1), 0, n, n)) * temp;
        Y = Y + temp;
    end
    output = 2 * Y / norm(b, 'fro')^2;
return

